"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RSCClientReferenceManifestRspackPlugin = void 0;
const path_1 = __importDefault(require("path"));
const binding_1 = require("@rspack/binding");
const base_1 = require("./base");
const RawRSCClientReferenceManifestRspackPlugin = (0, base_1.create)(binding_1.BuiltinPluginName.RSCClientReferenceManifestRspackPlugin, () => { }, "compilation");
class RSCClientReferenceManifestRspackPlugin {
    constructor(options = {}) {
        this.plugin = new RawRSCClientReferenceManifestRspackPlugin();
        this.options = options;
        this.resolvedOptions = {};
    }
    apply(compiler) {
        this.plugin.apply(compiler);
        this.resolvedOptions = this.resolveOptions(compiler);
        if (!compiler.options.module.rules) {
            compiler.options.module.rules = [];
        }
        compiler.options.module.rules.push({
            test: [/\.(j|t|mj|cj)sx?$/i],
            exclude: this.options.exclude ?? {
                // Exclude libraries in node_modules ...
                and: [/node_modules/]
            },
            use: [
                {
                    loader: "builtin:rsc-client-entry-loader",
                    options: this.resolvedOptions
                }
            ]
        });
    }
    resolveOptions(compiler) {
        const entry = Object.assign({}, compiler.options.entry);
        const resolvedEntry = {};
        const root = compiler.options.context ?? process.cwd();
        // TODO: support dynamic entry
        if (typeof entry === "object") {
            for (let item of Object.keys(entry)) {
                const imports = entry[item].import;
                if (imports) {
                    resolvedEntry[item] = imports[0];
                }
            }
        }
        const resolvedRoutes = this.options.routes ?? [];
        // TODO: config output
        const output = path_1.default.resolve(root, "./dist/server");
        return {
            entry: resolvedEntry,
            root: output,
            routes: resolvedRoutes
        };
    }
}
exports.RSCClientReferenceManifestRspackPlugin = RSCClientReferenceManifestRspackPlugin;
